"""Type stubs for translit._enums."""

import enum

class Script(enum.Enum):
    # Major world scripts
    LATIN = ...
    CYRILLIC = ...
    GREEK = ...
    ARABIC = ...
    HEBREW = ...
    # Indic scripts
    DEVANAGARI = ...
    BENGALI = ...
    GURMUKHI = ...
    GUJARATI = ...
    ORIYA = ...
    TAMIL = ...
    TELUGU = ...
    KANNADA = ...
    MALAYALAM = ...
    SINHALA = ...
    # East Asian scripts
    HAN = ...
    HIRAGANA = ...
    KATAKANA = ...
    HANGUL = ...
    # Southeast Asian scripts
    THAI = ...
    LAO = ...
    MYANMAR = ...
    KHMER = ...
    BALINESE = ...
    JAVANESE = ...
    TAI_LE = ...
    NEW_TAI_LUE = ...
    # Central/North Asian scripts
    TIBETAN = ...
    MONGOLIAN = ...
    # Caucasian scripts
    GEORGIAN = ...
    ARMENIAN = ...
    # African scripts
    ETHIOPIC = ...
    NKO = ...
    VAI = ...
    # Middle Eastern scripts
    SYRIAC = ...
    THAANA = ...
    COPTIC = ...
    # Americas
    CHEROKEE = ...
    CANADIAN_ABORIGINAL = ...
    # Historical European scripts
    RUNIC = ...
    OGHAM = ...
    GOTHIC = ...
    # Ancient Near Eastern scripts
    OLD_PERSIAN = ...
    CUNEIFORM = ...
    LINEAR_B = ...
    # Meta-scripts
    COMMON = ...
    INHERITED = ...

# European languages
LANG_BG: str
LANG_CA: str
LANG_CS: str
LANG_CY: str
LANG_DA: str
LANG_DE: str
LANG_EL: str
LANG_ES: str
LANG_ET: str
LANG_FI: str
LANG_FR: str
LANG_GA: str
LANG_HR: str
LANG_HU: str
LANG_IS: str
LANG_IT: str
LANG_LT: str
LANG_LV: str
LANG_MT: str
LANG_NL: str
LANG_NO: str
LANG_PL: str
LANG_PT: str
LANG_RO: str
LANG_SK: str
LANG_SL: str
LANG_SQ: str
LANG_SR: str
LANG_SV: str
LANG_TR: str
LANG_UK: str
LANG_VI: str
# Non-European languages
LANG_AR: str
LANG_FA: str
LANG_JA: str
LANG_KO: str
LANG_RU: str
LANG_ZH: str
